program Virtual_Machine_USB_Boot;

uses
   Forms,
   ShellApi,
   Windows,
   ProcessViewer,
   Messages,
   SysUtils,
   Math,
   TNTSysutils,
   Classes,
   Dialogs,
   MainForm in 'MainForm.pas' {frmMain},
   AddEdit in 'AddEdit.pas' {frmAddEdit},
   Progress in 'Progress.pas' {frmProgress},
   Options in 'Options.pas' {frmOptions},
   InfoMess in 'InfoMess.pas' {frmInfoMess};

{$R *.res}

type
   TPWideCharArray = array[0..0] of PWideChar;

function CheckTokenMembership(TokenHandle: THandle; SidToCheck: PSID; var IsMember: BOOL): BOOL; stdcall; external advapi32;

function IsNotUserAdmin: Boolean;
const
   SECURITY_NT_AUTHORITY: TSIDIdentifierAuthority = (Value: (0, 0, 0, 0, 0, 5));
   SECURITY_BUILTIN_DOMAIN_RID = $00000020;
   DOMAIN_ALIAS_RID_ADMINS = $00000220;
var
   b: BOOL;
   AdministratorsGroup: PSID;
begin
   b := AllocateAndInitializeSid(
      SECURITY_NT_AUTHORITY,
      2, //2 sub-authorities
      SECURITY_BUILTIN_DOMAIN_RID, //sub-authority 0
      DOMAIN_ALIAS_RID_ADMINS, //sub-authority 1
      0, 0, 0, 0, 0, 0, //sub-authorities 2-7 not passed
      AdministratorsGroup);
   if (b) then
   begin
      if not CheckTokenMembership(0, AdministratorsGroup, b) then
         b := False;
      FreeSid(AdministratorsGroup);
   end;

   Result := not b;
end;

function SHGetFolderPathW(Ahwnd: HWND; Csidl: longint; Token: THandle; Flags: DWord; Path: PWideChar): HRESULT; stdcall; external 'SHFolder.dll';

function GetSpecialFolderPath(folder: integer): WideString;
const
   SHGFP_TYPE_CURRENT = 0;
var
   path: array[0..MAX_PATH] of WideChar;
begin
   try
      if SUCCEEDED(SHGetFolderPathW(0, folder, 0, SHGFP_TYPE_CURRENT, @path[0])) then
         Result := path
      else
         Result := '';
   except
      Result := '';
   end;
end;

const
   CSIDL_APPDATA = $001A;
var
   i, j: Integer;
   NumArgs: Integer;
   TempArgs: PPWideChar;
   ParamStrW: array of WideString;
   ParamCountW: Integer;
   Appdata: WideString;
   StartupInfo: TStartupInfo;
begin
   Application.Initialize;
   Application.Title := 'Virtual Machine USB Boot';
   isInstalledVersion := False;

   if CreateFileMapping($FFFFFFFF, nil, PAGE_READONLY, 0, 32, 'VirtualMachineUSBBoot') <> 0 then
   begin
      if GetLastError = ERROR_ALREADY_EXISTS then
      begin
         GetProcessList;
         for i := 0 to High(ProcessInfo) do
            if ProcessInfo[i].WClass = 'TApplication' then
               if ProcessInfo[i].WCaption = 'Virtual Machine USB Boot' + appVersion then
                  if ProcessInfo[i].Handle <> Application.Handle then
                  begin
                     if IsIconic(ProcessInfo[i].Handle) then
                        SendMessage(ProcessInfo[i].Handle, WM_SYSCOMMAND, SC_RESTORE, 0);
                     SetForegroundWindow(ProcessInfo[i].Handle);
                     Application.Terminate;
                     Exit;
                  end;
      end;
   end;

   if IsNotUserAdmin then
   begin
      MessageBox(Application.Handle, 'In order to be able to properly detect and use the computer''s capabilities,'#13#10'this application requires administrator privileges.'#13#10'You can start it directly with "Run as administrator" or you can modify the shortcut''s properties.', 'Warning', MB_OK or MB_ICONWARNING);
      Exit;
   end;

   SetLength(ParamStrW, 0);
   ParamCountW := -1;
   TempArgs := nil;
   try
      TempArgs := CommandLineToArgvW(GetCommandLineW, NumArgs);
   except
   end;
   if TempArgs <> nil then
   begin
      SetLength(ParamStrW, NumArgs);
      ParamCountW := NumArgs - 1;
      try
         for i := 0 to NumArgs - 1 do
            ParamStrW[i] := TPWideCharArray(TempArgs^)[i];
      except
      end;
      try
         LocalFree(THandle(TempArgs));
      except
      end;
   end;
   if ParamCount > ParamCountW then
   begin
      SetLength(ParamStrW, ParamCount + 1);
      for i := ParamCountW + 1 to ParamCount do
         ParamStrW[i] := ParamStr(i);
   end;
   if isInstalledVersion then
   begin
      Appdata := GetSpecialFolderPath(CSIDL_APPDATA);
      if Appdata = '' then
         VMentriesFile := ParamStrW[0]
      else
         VMentriesFile := Appdata + '\' + WideChangeFileExt(WideExtractFileName(ParamStrW[0]), '') + '\' + WideExtractFileName(ParamStrW[0]);
   end
   else
      VMentriesFile := ParamStrW[0];
   VMentriesFile := WideChangeFileExt(VMentriesFile, '');
   CfgFile := VMentriesFile + '.cfg';
   VMentriesFile := VMentriesFile + '.vml';

   Application.ShowMainForm := False;
   Application.CreateForm(TfrmMain, frmMain);
   with frmMain do
   begin
      MainLeft := (Screen.WorkAreaWidth - Width) div 2 + Screen.WorkAreaLeft;
      MainTop := (Screen.WorkAreaHeight - Height) div 2 + Screen.WorkAreaTop;
      MainWidth := Width;
      MainHeight := Height;
      MainMaximized := WindowState = wsMaximized;
      mmCrt.Tag := lvVms.Canvas.TextWidth(IntToStr(Length(VMentries))) + 2 * lvVms.Canvas.TextWidth('0');
      GetStartUpInfo(StartupInfo);
      if WideFileExists(CfgFile) and LoadCfg(CfgFile) then
      begin
         if AddSecondDrive then
         begin
            pmHeaders.Items[4].Visible := True;
            pmHeaders.Items[3].Caption := 'First drive';
         end;
         lvVMs.Items.BeginUpdate;
         lvVMs.Columns.BeginUpdate;
         try
            for i := lvVMs.Columns.Count - 1 downto 1 do
               lvVMs.Columns[i].Free;
            for i := 0 to lvVMS.Items.Count - 1 do
               lvVMS.Items[i].SubItems.Clear;
            for i := 0 to pmHeaders.Items.Count - 1 do
               if pmHeaders.Items[i].Checked and pmHeaders.Items[i].Visible then
                  with lvVMS.Columns.Add do
                  begin
                     Caption := pmHeaders.Items[i].Caption;
                     MinWidth := 50;
                     Tag := i;
                     case Tag of
                        0:
                           begin
                              Alignment := taCenter;
                              pmHeaders.Items[i].Tag := lvVms.Canvas.TextWidth(IntToStr(Length(VMentries)) + '00');
                              MinWidth := pmHeaders.Items[i].Tag;
                              MaxWidth := pmHeaders.Items[i].Tag;
                              if lvVMs.Items.Count < 10 then
                              begin
                                 for j := 0 to lvVMS.Items.Count - 1 do
                                    lvVMS.Items[j].SubItems.Add(IntToStr(j + 1));
                              end
                              else
                                 for j := 0 to lvVMS.Items.Count - 1 do
                                    if j < 9 then
                                       lvVMS.Items[j].SubItems.Add('0' + IntToStr(j + 1))
                                    else
                                       lvVMS.Items[j].SubItems.Add(IntToStr(j + 1));
                           end;
                        1:
                           begin
                              for j := 0 to High(VMEntries) do
                                 lvVMs.Items[j].SubItems.Add(VMEntries[j].Name);
                           end;
                        2:
                           begin
                              ImageIndex := 3;
                              for j := 0 to High(VMEntries) do
                                 lvVMs.Items[j].SubItems.Add(VMEntries[j].VMName);
                           end;
                        3:
                           begin
                              ImageIndex := 2;
                              for j := 0 to High(VMEntries) do
                                 lvVMs.Items[j].SubItems.Add(VMEntries[j].FirstDriveName);
                           end;
                        4:
                           begin
                              ImageIndex := 2;
                              for j := 0 to High(VMEntries) do
                                 lvVMs.Items[j].SubItems.Add(VMEntries[j].SecondDriveName);
                           end;
                     end;
                     Width := pmHeaders.Items[i].Tag;
                  end;
         except
         end;
         lvVMs.Columns.EndUpdate;
         lvVMS.Items.EndUpdate;
         lvVms.ItemIndex := LastSelected;
         if LastSelected > -1 then
         begin
            lvVMs.Items[LastSelected].Selected := True;
            lvVMs.Items[LastSelected].Focused := True;
            lvVMs.Items[LastSelected].MakeVisible(False);
         end;
      end
      else
      begin
         if lvVMs.Columns.Count > 1 then
            if lvVMs.Columns[1].Tag = 0 then
            begin
               lvVMs.Columns[1].MinWidth := mmCrt.Tag;
               lvVMs.Columns[1].MaxWidth := mmCrt.Tag;
               lvVMs.Columns[1].Width := mmCrt.Tag;
            end;
      end;
   end;
   Application.OnActivate := frmMain.AppActDeactModBegModEnd;
   Application.OnDeactivate := frmMain.AppActDeactModBegModEnd;
   Application.OnModalBegin := frmMain.AppActDeactModBegModEnd;
   Application.OnModalEnd := frmMain.AppActDeactModBegModEnd;
   Application.OnException := frmMain.OnException;
   Application.ShowMainForm := True;
   with frmMain do
   begin
      SetBounds(MainLeft, MainTop, MainWidth, MainHeight);
      IntLeft := Left;
      IntTop := Top;
      IntWidth := Width;
      IntHeight := Height;
      try
         if MainMaximized or (((StartupInfo.dwFlags or STARTF_USESHOWWINDOW) = StartupInfo.dwFlags) and (StartupInfo.wShowWindow = SW_MAXIMIZE)) then
         begin
            if not (((StartupInfo.dwFlags or STARTF_USESHOWWINDOW) = StartupInfo.dwFlags) and (StartupInfo.wShowWindow in [SW_MINIMIZE, SW_SHOWMINNOACTIVE])) then
            begin
               frmMain.Perform(WM_SYSCOMMAND, SC_MAXIMIZE, 0);
               SetActiveWindow(Application.Handle);
            end;
         end;
      except
      end;
   end;
   SaveMaximized := MainMaximized or (not (((StartupInfo.dwFlags or STARTF_USESHOWWINDOW) = StartupInfo.dwFlags) and (StartupInfo.wShowWindow = SW_MAXIMIZE)));
   Application.Run;
end.

